package Mailer;

use MIME::Base64;
use Encode qw/from_to/;
use Encode::Guess qw(euc-jp shiftjis 7bit-jis);
use Net::SMTP;

# use warnings;
use Carp qw(croak);

use vars qw($config);

my $VERSION=1.0;

####
sub new {
	my $class=shift;
	my $self={
			'to'=>'',
			'from'=>'',
			'cc'=>'',
			'replyto'=>'',
			'account'=>'',
			'password'=>'',
			'sender'=>'/usr/sbin/sendmail',
			'subject'=>'mail from form',
			'body'=>'',
			'template'=>'',
			'charset'=>'ISO-2022-JP',
			'smtp'=>1,
			'debug'=>0,
			@_
		};

	bless($self, $class);

	$config=1;

	return $self;
}# new

####
sub version {
	$VERSION;
}# version

####
sub Define {
	my $self=shift;
	my %arg=(@_);
	foreach  my $key(keys %arg){
		$self->{$key}=$arg{$key} unless defined $self->{$key};
	}
}# define

####
sub Mailto {
	my $self=shift;
	$self->{'to'}=shift if(@_);
	$self->{'to'}=~s/\r|\n//g;
	$self->{'to'};
}# Mailto

####
sub From {
	my $self=shift;
	$self->{'from'}=shift if(@_);
	$self->{'from'}=~s/\r|\n//g;
	$self->{'from'};
}# From

sub Replyto {
	my $self=shift;
	$self->{'replyto'}=shift if(@_);
	$self->{'replyto'}=~s/\r|\n//g;
	$self->{'replyto'};
}# From


####
sub Cc {
	my $self=shift;
	$self->{'cc'}=shift if(@_);
	$self->{'cc'}=~s/\r\n/;/g;
	$self->{'cc'}=~s/\r|\n/;/g;
	$self->{'cc'};
}# Cc

####
sub Bcc {
	my $self=shift;
	$self->{'bcc'}=shift if(@_);
	$self->{'bcc'}=~s/\r\n/;/g;
	$self->{'bcc'}=~s/\r|\n/;/g;
	$self->{'bcc'};
}# Cc

####
sub Certify {
	my $self=shift;
	$self->{'account'}=shift;
	$self->{'password'}=shift;

	$self->{'account'}=~s/\r|\n//g;
	$self->{'password'}=~s/\r|\n//g;

	$self;
}# Certify


####
sub Charset{
	my $self=shift;
	$self->{'charset'}=shift if(@_);
	$self->{'charset'};
}

####
sub Debug{
	my $self=shift;
	
	$self;
}# Debug

####
sub Smtp {
	my $self=shift;
	$self->{'smtp'}=shift if(@_);
	$self->{'smtp'};
}

####
sub Subject {
	my $self=shift;
	my $subject=shift;

	if(defined $subject){
		$subject=~s/\r|\n//g;

		my $enc=Encode::Guess::guess_encoding($subject, qw/euc-jp shiftjis 7bit-jis/);

		if($self->{'charset'} eq 'ISO-2022-JP' and  ($enc->name ne 'ISO-2022-JP' or $enc->name ne ('7bit-jis'))){
			Encode::from_to($subject, $enc->name, '7bit-jis');
			$subject=encode_base64($subject);
			chomp($subject);
			$self->{'subject'}="=?ISO-2022-JP?B?$subject?=";
		}else{
			$self->{'subject'}=$subject;
		}
	}

	$self->{'subject'};
}# Subject


#### set/get Template file name
sub Template {
	my $self=shift;
	$fname=shift if(@_);

	my $tpl;
# print "Template", $fname;
	if($fname and (-e $fname)){
		open(FP, '<', $fname);
# print "fp", FP, "\n";

		if(FP){
# print "Open OK $fname\n";

			local $/=undef;
			$self->{'template'}=<FP>;
			close FP;

			my $enc=Encode::Guess::guess_encoding($self->{'template'}, qw/euc-jp shiftjis 7bit-jis/);
			Encode::from_to($self->{'template'}, $enc->name, 'utf-8') if($enc->name ne 'utf-8' or $enc->name ne 'ascii');

		}else{
# print "Open error $fname\n";
		}

	}
	
	$self->{'template'};
}# Template

#### set body from argument and template file
sub Body {
	my $self=shift;
	my %qs=(@_);

	return $self->{'body'} unless(%qs);

	my $message=$self->{'body'}||$self->{'template'};
	

{
	my $tmp='';
	foreach my $key(keys %qs){
		$tmp.=$qs{$key};
	}
	my $enc=Encode::Guess::guess_encoding($tmp, qw/euc-jp shiftjis 7bit-jis/);
	my $val;
	foreach my $key(keys %qs){
		from_to($qs{$key}, $enc->name, 'utf-8') if($enc->name ne 'utf-8');
		$message=~s/<$key>/$qs{$key}/g;
	}
}
	
	$self->{'body'}=$message;

	$self;
}# Apply

####
sub Reset {
	my $self=shift;
	$self->{'body'}='';
}

####
sub Print {
	my $self=shift;
	print $self->{'body'};
}

####
sub Send {
	my $self=shift;

	my $sender=$self->{'sender'};
	my $mailfrom=$self->{'from'};

	my $fname;

# 'to'、'body'がなければ送信しない
	return 'To is null'  unless($self->{'to'});
	return 'Body is null' if($self->{'body'} eq '');

## for outlook6
	my $CR=chr(13);
	my $LF=chr(10);
	my $CRLF=$CR.$LF;
	my $body=$self->{'body'};
	$body=~s/$CRLF/$LF/g;
	$body=~s/$CR/$LF/g;
## for outlook6 end

	if($self->{'charset'} eq 'ISO-2022-JP'){
		Encode::from_to($body, 'utf-8', '7bit-jis');
	}
	
	if($self->{'debug'}){
		$fname = '>mailto'.$self->{'debug'}.'.txt';
	}elsif($self->{'smtp'}){
		# nothing
	}else{
		$fname= qq{| $sender -t -f$mailfrom};
	}

	if($self->{'debug'} or !($self->{'smtp'}) ){
		if(open(MAIL, $fname)){
			print MAIL 'To: ',$self->{'to'},"\n";
			print MAIL 'From: ',$self->{'from'},"\n";
			print MAIL 'Cc: ',$self->{'cc'},"\n" if($self->{'cc'});
			print MAIL 'Bc: ',$self->{'bcc'},"\n" if($self->{'bcc'});
			print MAIL 'Reply-To: ',$self->{'replyto'},"\n" if($self->{'replyto'});
			print MAIL 'Subject: ',$self->{'subject'},"\n";
			
			if($self->{'charset'} eq 'ISO-2022-JP'){
				print MAIL 'Content-Transfer-Encoding: 7bit',"\n";
				print MAIL 'Content-Type: text/plain; charset=iso-2022-jp', "\n";
			}
			print MAIL "\n";
			print MAIL $body;
			print MAIL "\n";
			close MAIL;
			
			return ($self->{'debug'})?$fname:1;
		}else{
			return 'open error '.$fname;
		}

	}else{ # if($self->{'debug'} or !($self->{'smtp'}))
		my $smtp = Net::SMTP->new('localhost', Hello=>"localhost");
		if($smtp){
			# 送信元の指定
			$smtp->mail($self->{'from'});
			# 宛先の指定
			$smtp->to($self->{'to'});
			# ヘッダ
			$smtp->data();
			$smtp->datasend('From: '.$self->{'from'}."\n");
			$smtp->datasend('To: '.$self->{'to'}."\n");
			$smtp->datasend('Cc: '.$self->{'cc'}."\n") if($self->{'cc'});
			$smtp->datasend('Bcc: '.$self->{'bcc'}."\n") if($self->{'bcc'});
			$smtp->datasend('Reply-To: '.$self->{'replyto'}."\n") if($self->{'replyto'});
			$smtp->datasend('Subject: '.$self->{'subject'}."\n");

			if($self->{'charset'} eq 'ISO-2022-JP'){
				$smtp->datasend("Content-Transfer-Encoding: 7bit\n");
				$smtp->datasend("Content-Type: text/plain;charset=\"ISO-2022-JP\"\n\n");
			}

			# 本文
			$smtp->datasend("$body\n");
			# データの終わり、メール送信
			$smtp->dataend();
			# SMTP接続の終了
			$smtp->quit;
			return 0;
		}else{
			return 'smtp error';
		}
	}

}# Send

1;

